unit OptEls;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//    Visual Component for Drawing Optical Elements (Lenses and Mirrors)      //
//                            (Delphi Release)                                //
//                                                                            //
//                              Name: TOptEls                                 //
//                     Version: 2.00 (Delphi Release)                         //
//                        Author: Andrey G. Sadovoy                           //
//                                aug. 2008                                   //
//                                                                            //
//   e-mail: sadovoya@mtu-net.ru                                              //
//   http://SintResEng.narod.ru (English site)                                //
//   http://SadovoyA.optelsnarod.ru (Russian site)                            //
//   freeware, open source, 'as is'                                           //
////////////////////////////////////////////////////////////////////////////////

interface

uses
  Windows, Classes, Controls, Graphics;

type
  TKindSet = (pLens, nLens,flMirror,// possible elements kinds
             pMirrorR, pMirrorL,
             nMirrorR, nMirrorL);

  TCurvFromWidth = 0..50; // accessible ledges from width in percentage

  TOptEls = class(TGraphicControl)
  private
    { Private declarations }
  FXPos:Integer;  //   element centre position
  FYPos: Integer; //   (by X and Y)
  FCurvFromWidth: TCurvFromWidth; // curvature
  FOnKindChange: TNotifyEvent;  // element Kind change event
  FOnXPosChange: TNotifyEvent; // x position change event
  FOnYPosChange: TNotifyEvent; // the same for Y
  FKind : TKindSet;//kind of an optical element
  FPix: Array [0..21] of TPoint; // points of contour of an element
  FBodyColor: TColor; // colour of filling or shading
  FContourColor: TColor;  // colour of contour
  FBodyStyle: TBrushStyle; // style of filling of element body
  FContourWidth: Integer; // thickness of a contour line
  FContourStyle: TPenStyle; // contour line style
  procedure CreateFPix; // formation of contour points
  function GetKind: TKindSet;
  procedure SetKind(AValue: TKindSet);
  function GetXPos: Integer;
  procedure SetXPos(AValue:Integer);
  function GetYPos: Integer;
  procedure SetYPos(AValue:Integer);
  function GetCurvFromWidth: TCurvFromWidth;
  procedure SetCurvFromWidth(AValue: TCurvFromWidth);
  procedure SetBodyColor(AValue: TColor);
  procedure SetContourColor(AValue: TColor);
  procedure SetBodyStyle(AValue: TBrushStyle);
  procedure SetContourWidth(AValue: Integer);
  procedure SetContourStyle(AValue: TPenStyle);

  protected
    { Protected declarations }

  public

  constructor Create(AOwner:TComponent); override;
  procedure Paint; override; // let's draw

  published

  property OnMouseDown; //We publish in the inspector
  property OnClick;     //necessary events
  property PopUpMenu;   // from an ancestor
  property Caption;

  // we add ours :
  property Kind: TKindSet
  read GetKind
  write SetKind;   //for display in the inspector

  property OnKindChange: TNotifyEvent
  read FOnKindChange
  write FOnKindChange;

  property XPos: integer
  read GetXPos
  write SetXPos;

  property YPos: Integer
  read GetYPos
  write SetYPos;

  property OnXPosChange: TNotifyEvent
  read FOnXPosChange
  write FOnXPosChange;

  property OnYPosChange: TNotifyEvent
  read FOnYPosChange
  write FOnYPosChange;

  property CurvFromWidth: TCurvFromWidth
  read GetCurvFromWidth
  write SetCurvFromWidth;

  property BodyColor: TColor
  read FBodyColor
  write SetBodyColor;

  property ContourColor: TColor
  read FContourColor
  write SetContourColor;

  property BodyStyle: TBrushStyle
  read FBodyStyle
  write SetBodyStyle;

  property ContourWidth: Integer
  read FContourWidth
  write SetContourWidth;

  property ContourStyle: TPenStyle
  read FContourStyle
  write SetContourStyle;

end;

procedure Register;



implementation



procedure Register;
begin
RegisterComponents('Optics', [TOptEls]);
end;

Constructor TOptEls.Create(AOwner:TComponent);
begin
Inherited Create(AOwner);
FCurvFromWidth:=25;
FKind:=pLens;
FBodyColor:=clBlue; Canvas.Brush.Color:=FBodyColor;
FContourColor:=clBlack; Canvas.Pen.Color:=FContourColor;
FBodyStyle:=bsSolid; Canvas.Brush.Style:=FBodyStyle;
FContourWidth:=0; Canvas.Pen.Width:=FContourWidth;
FContourStyle:=psSolid; Canvas.Pen.Style:=FContourStyle;
repaint;
end;

procedure TOptEls.SetBodyColor(AValue: TColor);
begin
FBodyColor:=AValue;
canvas.Brush.Color:=FBodyColor;
repaint;
end;

procedure TOptEls.SetContourColor(AValue: TColor);
begin
FContourColor:=AValue;
Canvas.Pen.Color:=FContourColor;
repaint;
end;

procedure TOptEls.SetBodyStyle(AValue: TBrushStyle);
begin
FBodyStyle:=AValue;
Canvas.Brush.Style:=FBodyStyle;
repaint;
end;

procedure TOptEls.SetContourWidth(AValue: Integer);
begin
FContourWidth:=AValue;
Canvas.Pen.Width:=FContourWidth;
repaint;
end;

procedure TOptEls.SetContourStyle(AValue: TPenStyle);
begin
FContourStyle:=AValue;
Canvas.Pen.Style:=FContourStyle;
repaint;
end;

function TOptEls.GetCurvFromWidth: TCurvFromWidth;
begin
Result:=FCurvFromWidth;
end;

procedure TOptEls.SetCurvFromWidth(AValue: TCurvFromWidth);
begin
FCurvFromWidth:=AValue;
repaint;
end;

Procedure TOptEls.CreateFPix;  // Calculations of contours
var
I: Word;
begin
  For I:=0 to 5 do
  begin   // the left concave parabola
  FPix[i].Y:=I*(Height-1) div 10;
  FPix[10-i].Y:=(Height-1)-FPix[i].Y; // for symmetry
  FPix[i].X:=4*(Width)*(-(FPix[i].Y)*FPix[i].Y +(Height-1)*FPix[i].Y)
  *FCurvFromWidth;
  FPix[i].X:=FPix[i].X div (100*(Height-1)*(Height-1));
  FPix[10-i].X:=FPix[i].X;
  end;
  for i:=0 to 10 do FPix[11+i].Y:=FPix[10-i].Y; // common
  case FKind of
  nlens: For I:= 0 to 10 do FPix[11+i].X:=-FPix[10-i].X+(Width-1);
  pMirrorR: for I:=0 to 10 do FPix[11+i].X:=Width-1;
  pMirrorL: for I:=0 to 10 do begin
            FPix[11+i].X:=-FPix[10-i].X+Width-1;
            FPix[10-i].X:=0;
            end;
  pLens: for I:= 0 to 10 do begin
         FPix[i].X:=-FPix[i].X + ((Width-1)*FCurvFromWidth) div 100;
         FPix[11+i].X:=-FPix[i].X+ Width-1 ;
         end;
  nMirrorR: for I:= 0 to 10 do begin
            FPix[i].X:=-FPix[i].X + ((Width-1)*FCurvFromWidth) div 100;
            FPix[11+i].X:=Width-1;
            end;
  nMirrorL: for I:=0 to 10 do begin
            FPix[11+i].X:=FPix[i].X+(Width-1)
            -((Width-1)*FCurvFromWidth) div 100;
            FPix[i].X:=0;
            end;
 end;
end;

procedure TOptEls.Paint;
var
 i: Word;
 Rect: TRect;
begin
If (Height <= 1) then exit;
if FKind = flMirror then
begin
  Rect.TopLeft.X:= ContourWidth div 2;
  Rect.TopLeft.Y:= ContourWidth div 2;
  Rect.BottomRight.X:= Width-ContourWidth div 2;
  Rect.BottomRight.Y:= Height-1-ContourWidth div 2;
  Canvas.Rectangle(Rect); exit;
end;
CreateFPix;
for i := 0 to 10 do
begin
  FPix[i].X:=FPix[i].X + ContourWidth div 2;
  FPix[i+11].X:=FPix[i+11].X - ContourWidth div 2;
end;
FPix[0].Y:=ContourWidth div 2;
FPix[21].Y:=ContourWidth div 2;
FPix[10].Y:=Height-1-ContourWidth div 2;
FPix[11].Y:=Height-1-ContourWidth div 2;
Canvas.Polygon(FPix); 
end;

function TOptEls.GetKind: TKindSet;
begin
Result:=FKind;
end;

Procedure TOptEls.SetKind(AValue: TKindSet);
begin
FKind:=AValue;
repaint;
If Assigned(FOnKindChange) then OnKindChange(self);
end;

function TOptEls.GetXPos: Integer;
begin
Result:=Left+Width div 2;
end;

procedure TOptEls.SetXPos(AValue:Integer);
begin
FXPos:=AValue;
Left:=FXPos - Width div 2;
repaint;
If Assigned(FOnXPosChange) then OnXposChange(self)
end;

function TOptEls.GetYPos: Integer;
begin
Result:=Top+Height div 2;
end;

procedure TOptEls.SetYPos(AValue:Integer);
begin
FYPos:=AValue;
Top:=FYPos - Height div 2;
repaint;
If Assigned(FOnYPosChange) then OnYPosChange(self) 
end;

end.

